public class JBinomTable {


	int    n;
	Double p;

	Double[] bincoeff;				// nCr values
	Double[] binvalues;				// single Bernoulli values
	Double[] cumulated;				// cumulated values


	
	public JBinomTable(int nn, Double pp) {

		bincoeff  = new Double[ 101 ];	// maximum: n = 100 (0..100)
		binvalues = new Double[ 101 ];
		cumulated = new Double[ 101 ];

		reCalc(nn, pp);

	}

	public void reCalc(int nn, Double pp) {

		n = nn; p = pp;
		
		calcCoeff(); calcValues();

	}

	public void reCalcP(Double pp) {

		p = pp; calcValues();

	}

	public void reCalcN(int nn) {

		n = nn; calcCoeff(); calcValues();

	}

	// get() methods for values and both tables:

	public int getN() {

		return n;

	}

	public Double getP() {

		return p;

	}

	public Double getSingle(int k) {

		return binvalues[k];

	}
	
	public Double getCumulated(int k) {

		return cumulated[k];

	}

	public Double getInterval(int k1, int k2) {

		Double pInterval = 0.0;
		
		if ( k1 == 0 ) {
			pInterval = cumulated[k2];
		} else {
			pInterval = cumulated[k2] - cumulated[k1-1];
		}
		
		return pInterval;

	}

	public Double getMu() {

		return 1.0*n*p;

	}

	public Double getSigma() {

		return Math.sqrt( 1.0*n*p*(1-p) );

	}


	// Trick: nC(k+1) = nCk * (n-k) / (k+1)

	private void calcCoeff() {

		bincoeff[0] = 1.0;
		
		for (int i=1; i <=n; i++) {
			bincoeff[i] = ( bincoeff[i-1] * (n-i+1) ) / i;
		}

	}

	private void calcValues() {

		// start in the "middle" of the table at µ = n*p:

		int mid = (int)Math.round( n * p );
		Double midPow = Math.pow(p, mid) * Math.pow(1-p, n-mid);
		binvalues[mid] = bincoeff[mid] * midPow;

		// recalculate the powers whithout Math.pow() to the right...

		Double stepPow = midPow;
		for (int i=mid+1; i<=n; i++) {
			stepPow = stepPow * p / (1-p);
			binvalues[i] = bincoeff[i] * stepPow;
		}

		for (int i=n+1; i<101; i++) {
			binvalues[i] = 0.0;
		}

		// ... and to the left

		stepPow = midPow;
		for (int i=mid-1; i>=0; i--) {
			stepPow = stepPow * (1-p) / p;
			binvalues[i] = bincoeff[i] * stepPow;
		}

		// sum up all values for cumulated table

		cumulated[0] = binvalues[0];
		
		for (int i=1; i<=n; i++) {
			cumulated[i] = cumulated[i-1]+binvalues[i];
		}
		
	}

}
