import java.io.*;
import java.awt.*;
import java.awt.event.*;
import java.awt.image.*;
import javax.swing.*;
import javax.swing.event.*;
import javax.imageio.*;
import java.util.*;

class JBinomial extends JFrame 
	implements ActionListener, ChangeListener {

	JTabs		tabs;
	JPanel 		mainPanel;
	JDiagram 	diagram;
	JSliders 	sliders;
	JBottomBar  bottomBar;
	JInfoDialog infoDialog;
	
	private final Color myWhite = new Color(250,250,250);
 
    public JBinomial() {
		setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
		setTitle("JBinomial");
		getContentPane().setPreferredSize(new Dimension(1000, 700));

		try {
			Image icon = ImageIO.read(getClass().getResource("/pic/logo.png"));
			setIconImage(icon);
		} catch(IOException e) { e.printStackTrace();	}

		setupLayout();

		pack(); setLocationRelativeTo(null); setVisible(true);
		
		I18N messages = I18N.getInstance();
		infoDialog = new JInfoDialog(this);		

    }

	public void actionPerformed (ActionEvent event){

		if ( event.getSource() == sliders.pBox ) { 
			if ( sliders.isPboxValid() ) {
				sliders.setPfromBox = true;
				sliders.setPslider( sliders.getPbox() );
				diagram.updateP( sliders.getPbox(),
								 sliders.getPboxString() );
				bottomBar.setMuSigma(
				   diagram.binomTable.getMu(),
				   diagram.binomTable.getSigma() );
			}
		}

		if ( event.getSource() == sliders.nBox ) { 
			if ( sliders.isNboxValid() ) {
				sliders.setNfromBox = true;
				sliders.setNslider( sliders.getNbox() );
				diagram.updateN( sliders.getNbox() );
				bottomBar.setMuSigma(
				   diagram.binomTable.getMu(),
				   diagram.binomTable.getSigma() );
			}
		}

		if ( event.getSource() == bottomBar.tableButton ) { 
			diagram.showTable.showTable(
				sliders.pBox.textField.getText(), 
				sliders.nBox.textField.getText()
			);
		}

		if ( event.getSource() == bottomBar.fullButton ) { 
			diagram.setFull();
		}


		if ( event.getSource() == bottomBar.optButton ) { 
			diagram.setOptimum();
		}

		if ( event.getSource() == bottomBar.showButton ) { 
			diagram.switchShow( bottomBar.showButton.isSelected() );
		}

		if ( event.getSource() == bottomBar.hypoType ) { 
			diagram.setHypoMode( bottomBar.hypoType.getSelectedIndex() );
		}


		if ( event.getSource() == diagram.popup.copyItem ) { 
			diagram.copyToClipboard();
		}

		if ( event.getSource() == diagram.popup.exitItem ) { 
			System.exit(0);
		}

		if (event.getSource() == diagram.popup.infoItem ) {
			infoDialog.showInfo();
		}


		if ( event.getSource() == bottomBar.lrButton ) { 

			mainPanel.remove( sliders );

			if ( bottomBar.lrButton.isSelected() ) {
				mainPanel.add( sliders, BorderLayout.LINE_START );
			} else {
				mainPanel.add( sliders, BorderLayout.LINE_END );
			}

			sliders.reOrder(); bottomBar.changeLR();

			mainPanel.validate();

		}

  
	}

	public void stateChanged(ChangeEvent e) {

        if ( e.getSource() == sliders.pSlider ) {
			if (!sliders.setPfromBox) {
				diagram.updateP( sliders.getPslider(),
				                 sliders.getPsliderString() );
				sliders.setPbox();
				bottomBar.setMuSigma(
				   diagram.binomTable.getMu(), 
				   diagram.binomTable.getSigma() );
			} else { sliders.setPfromBox = false; }
			
		}

        if ( e.getSource() == sliders.nSlider ) {
			if (!sliders.setNfromBox) {
				diagram.updateN( sliders.getNslider() );
				sliders.setNbox();
				bottomBar.setMuSigma(
				   diagram.binomTable.getMu(),
				   diagram.binomTable.getSigma() );
			} else { sliders.setNfromBox = false; }
			
		}

        if ( e.getSource() == bottomBar.singleKslider ) {
			diagram.setShowKValues( bottomBar.getK1Slider(), 0);
		}

        if ( e.getSource() == bottomBar.sigmaSlider ) {
			diagram.setShowSigmaA( bottomBar.getSigmaAvalue() );
		}

        if ( e.getSource() == bottomBar.minMaxSlider ) {
			diagram.setShowKValues( 
			      bottomBar.getIntervalSliderMin(),
			      bottomBar.getIntervalSliderMax()    );
		}

        if ( e.getSource() == bottomBar.hypoSlider ) {
			diagram.setShowHypoAlpha( bottomBar.hypoSlider.getValueDouble() );
		}

        if ( e.getSource() == tabs ) {

			diagram.setShowMode( tabs.getSelectedIndex() );
			bottomBar.setPanel( tabs.getSelectedIndex() );

		}

    }


	public void setupLayout() {

		mainPanel = new JPanel( new BorderLayout() );

		diagram = new JDiagram(this, 20,0.5); 

		JPanel borderPanel = new JPanel( new BorderLayout() );
		
		borderPanel.setBackground( myWhite );
		borderPanel.setBorder( BorderFactory.createEmptyBorder(10,10,10,10) );
		borderPanel.add( diagram );

		mainPanel.add( borderPanel, BorderLayout.CENTER );
		
		sliders = new JSliders(this);    
		mainPanel.add( sliders, BorderLayout.EAST );
		
		tabs = new JTabs( mainPanel );
		
		tabs.addChangeListener(this);
		
		add( tabs, BorderLayout.CENTER );
		
		bottomBar = new JBottomBar( this );
		add( bottomBar, BorderLayout.SOUTH );

	}
	
 
    public static void main(String[] args) {

		System.setProperty("awt.useSystemAAFontSettings","on"); 
		System.setProperty("swing.aatext", "true"); 

		String os = System.getProperty("os.name");
		
		String ui = UIManager.getSystemLookAndFeelClassName();
		
		if ( os.startsWith("Linux") ) {
			ui = "com.sun.java.swing.plaf.gtk.GTKLookAndFeel";
		}

		if ( os.startsWith("Windows") ) {
			ui = "com.sun.java.swing.plaf.windows.WindowsLookAndFeel";
		}
		
		try {
			UIManager.setLookAndFeel(ui);
		} catch (UnsupportedLookAndFeelException e) {}
		  catch (ClassNotFoundException e) {}
		  catch (InstantiationException e) {}
		  catch (IllegalAccessException e) {}

		UIManager.put("Slider.paintValue", false);

		JBinomial f = new JBinomial();

    }

}
