import java.awt.*;
import javax.swing.*;
import javax.swing.table.*;

public class JBoldTable extends JTable {

	JFiraFont fira = JFiraFont.getInstance();
	
	private Font customFont;
	private int  customRowHeight;

	public JBoldTable( Object[][] data, Object[] columnNames ) {

		super( new DefaultTableModel(data, columnNames) );

		getColumnModel().getColumn(0).setMaxWidth(120);
		
		customFont = fira.font26;

		FontMetrics metrics = getFontMetrics(customFont);
		customRowHeight = metrics.getHeight()+10;
		setRowHeight(customRowHeight);

		getTableHeader().setFont( customFont );
		((DefaultTableCellRenderer)getTableHeader().getDefaultRenderer()).setHorizontalAlignment( SwingConstants.CENTER );

	}

	@Override
	public int getRowHeight() {
		if (customRowHeight > 0) {
			return customRowHeight;
		}
		return super.getRowHeight();
	}

	public boolean isCellEditable(int row, int column) {
		return false;               
	};

	public Component prepareRenderer(TableCellRenderer renderer, int row, int column) {
		Component c = super.prepareRenderer(renderer, row, column);
		if (customFont != null) {
			((JLabel)c).setHorizontalAlignment( SwingConstants.CENTER );
			c.setFont(customFont);
		}
		return c;
	}

	public void addData( String s1, String s2, String s3 ) {
		Object[] newRow = { s1, s2, s3 };

		((DefaultTableModel)getModel()).addRow( newRow );
	}

	public void clearData() {

		((DefaultTableModel)getModel()).setRowCount(0);
	}

}
